﻿using Microsoft.Extensions.Logging;
using PIbd_24_EredavkinRA_BusBusiness.Repositories;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PIbd_24_EredavkinRA_BusBusiness.Reports;

internal class ChartReport
{
    private readonly IToRepository _toRepository;

    private readonly ILogger<ChartReport> _logger;

    public ChartReport(IToRepository toRepository, ILogger<ChartReport> logger)
    {
        _toRepository = toRepository ??
            throw new ArgumentNullException(nameof(toRepository));
        _logger = logger ??
            throw new ArgumentNullException(nameof(logger));
    }

    public bool CreateChart(string filePath, DateTime dateTime)
    {
        try
        {
            new PdfBuilder(filePath)
               .AddHeader($"Количество Обслуживаний Автобусов на {dateTime:dd MMMM yyyy}")
                .AddPieChart("Автобусы", GetData(dateTime))
                .Build();

            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }

    private List<(string Caption, double Value)> GetData(DateTime dateTime)
    {
        return _toRepository
            .ReadTo(dateForm: dateTime.Date, dateTo: dateTime.Date.AddDays(1))
            .GroupBy(x => x.BusName, (key, cost) => new { BusName = key, Count = cost.Count() })
            .Select(x => (x.BusName, (double)x.Count))
            .ToList();
    }
}
