﻿using Microsoft.Extensions.Logging;
using PIbd_24_EredavkinRA_BusBusiness.Repositories;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PIbd_24_EredavkinRA_BusBusiness.Reports;

internal class DocReport
{
    private readonly IBusRepository _busRepository;

    private readonly IRoutRepositories _routRepository;

    private readonly IStaffRepositories _staffRepository;

    private readonly ILogger<DocReport> _logger;

    public DocReport(IBusRepository busRepository,
      IRoutRepositories routRepository, IStaffRepositories staffRepository, ILogger<DocReport> logger)
    {
        _busRepository = busRepository ??
            throw new ArgumentNullException(nameof(busRepository));
        _routRepository = routRepository ??
            throw new ArgumentNullException(nameof(routRepository));
        _staffRepository = staffRepository ??
            throw new ArgumentNullException(nameof(staffRepository));
        _logger = logger ??
            throw new ArgumentNullException(nameof(logger));
    }
    public bool CreateDoc(string filePath, bool includeBus, bool includeRout,
       bool includeStaff)
    {
        try
        {
            var builder = new WordBuilder(filePath)
                .AddHeader("Документ со справочниками");

            if (includeBus)
            {
                builder.AddParagraph("Автобусы")
                    .AddTable([1200, 2400, 1200], GetBuses());
            }

            if (includeRout)
            {
                builder.AddParagraph("Маршруты")
                .AddTable([2400, 2400, 2400], GetRout());
            }

            if (includeStaff)
            {
                builder.AddParagraph("Сотрудники")
                .AddTable([2400, 2400, 2400], GetStaff());
            }

            

            builder.Build();

            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }

    private List<string[]> GetBuses()
    {
        return [
            ["Модель", "Номерной знак", "Количество мест"],
            ..  _busRepository
                .ReadBus()
                .Select(x => new string[] { x.ModelType.ToString(), x.N_Z, x.Capacity.ToString() }),
        ];
    }

   

    private List<string[]> GetStaff()
    {
        return [
            ["Фамилия", "Имя", "Должность"],
            .. _staffRepository
                .ReadStaff()
                .Select(x => new string[] { x.LastName, x.FirstName, x.EmployeePost.ToString() }),
        ];
    }

    private List<string[]> GetRout()
    {
        return [
            ["Номер Маршрута", "Начальная остановка", "Конечная остановка"],
            .. _routRepository
                .ReadRout()
                .Select(x => new string[] { x.NumberRout.ToString(), x.StartStop, x.EndStop }),
        ];
    }
}
