﻿using Microsoft.Extensions.Logging;
using PIbd_24_EredavkinRA_BusBusiness.Entities;
using PIbd_24_EredavkinRA_BusBusiness.Repositories;
using PIbd_24_EredavkinRA_BusBusiness.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PIbd_24_EredavkinRA_BusBusiness.Reports;

internal class TableReport
{
    private readonly IRoutSheetRepository _routsheetRepository;

    private readonly IToRepository _toRepository;

    private readonly IBusRepository _busRepository;

    private readonly ILogger<TableReport> _logger;

    internal static readonly string[] item = ["Дата", "Номер Маршрута", "Стоимость ремонта"];

    public TableReport(IRoutSheetRepository routsheetRepository, IToRepository toRepository, IBusRepository busRepository, ILogger<TableReport> logger)
    {
        _routsheetRepository = routsheetRepository ??
            throw new ArgumentNullException(nameof(routsheetRepository));
        _toRepository = toRepository ??
            throw new ArgumentNullException(nameof(toRepository)); 
        _busRepository = busRepository ??
            throw new ArgumentNullException(nameof(busRepository));


        _logger = logger ??
            throw new ArgumentNullException(nameof(logger));
    }

    public bool CreateTable(string filePath, int busId, DateTime startDate, DateTime endDate)
    {
        try
        {
            new ExcelBuilder(filePath)
                .AddHeader("Общая сводка за Автобусами", 0, 3)
                .AddParagraph($"за период c {startDate:dd.MM.yyyy} по {endDate:dd.MM.yyyy}", 0)
                .AddTable([15, 10, 10], GetData(busId, startDate, endDate))
                .Build();

            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }

    public List<string[]> GetData(int busId, DateTime startDate, DateTime endDate)
    {
        var data = _routsheetRepository
            .ReadRoutSheet(dateForm: startDate, dateTo: endDate, busid: busId)
            .Select(x => new {
                Date = x.BusDate,
                Rout = x.RoutName ?? string.Empty,
                Cost = (decimal?)null
            })
            .Union(
                _toRepository
                    .ReadTo(dateForm: startDate, dateTo: endDate, busid: busId)
                    .Select(x => new {
                        Date = x.DateTo,
                        Rout = string.Empty,
                        Cost = (decimal?)x.Cost 
                    }))
            .OrderBy(x => x.Date);

        return new List<string[]> { item }
            .Union(
                data.Select(x => new string[]
                {
                x.Date.ToString("dd.MM.yyyy"),
                x.Rout?.ToString() ?? string.Empty,
                x.Cost?.ToString("N0") ?? string.Empty 
                }))
            .Union(new[] { new string[] { "Всего", "",  data.Sum(x => x.Cost ?? 0).ToString("N0") } }) 
            .ToList();

        
    }






}
