﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using PIbd_24_EredavkinRA_BusBusiness.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PIbd_24_EredavkinRA_BusBusiness.Repositories.Implementations;


public class BusRepository : IBusRepository
{
    private readonly IConnectionString _connectionString;

    private readonly ILogger<BusRepository> _logger;

    public BusRepository(IConnectionString connectionString, ILogger<BusRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateBus(Bus bus)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(bus));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
                INSERT INTO Bus (ModelType, N_Z, Capacity, Bus_mileage)
                VALUES (@ModelType, @N_Z, @Capacity, @Bus_mileage)";
            connection.Execute(queryInsert, bus);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }
    public void UpdateBus(Bus bus)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(bus));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"
                UPDATE Bus
                SET
                    ModelType=@ModelType, 
                    N_Z=@N_Z, 
                    Capacity=@Capacity, 
                    Bus_mileage=@Bus_mileage
                WHERE Id=@Id";
            connection.Execute(queryUpdate, bus);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }
    public void DeleteBus(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
                DELETE FROM Bus
                WHERE Id=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }
    public Bus ReadBusByID(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
                SELECT * FROM Bus
                WHERE Id=@id";
            var bus = connection.QueryFirst<Bus>(querySelect, new { id });
            _logger.LogDebug("Найденный объект: {json}", JsonConvert.SerializeObject(bus));
            return bus;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }

    public IEnumerable<Bus> ReadBus()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT * FROM Bus";
            var bus = connection.Query<Bus>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(bus));
            return bus;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }

  

   
}
