﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using PIbd_24_EredavkinRA_BusBusiness.Entities;


namespace PIbd_24_EredavkinRA_BusBusiness.Repositories.Implementations;

public class ToRepository : IToRepository
{
    private readonly IConnectionString _connectionString;

    private readonly ILogger<ToRepository> _logger;

    public ToRepository(IConnectionString connectionString, ILogger<ToRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateTo(To to)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(to));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
                INSERT INTO TOs (BusId, StaffId, Discription, Cost, DateTo)
                VALUES (@BusId, @StaffId, @Discription, @Cost, @DateTo)";
            connection.Execute(queryInsert, to);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

    public void DeleteTo(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
                DELETE FROM TOs
                WHERE Id=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public IEnumerable<To> ReadTo(DateTime? dateForm = null, DateTime? dateTo = null, int? staffid = null, int? busid = null)
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            var builder = new QueryBuilder();
            if (dateForm.HasValue)
            {
                builder.AddCondition("cs.DateTo >= @dateForm");
            }
            if (dateTo.HasValue)
            {
                builder.AddCondition("cs.DateTo <= @dateTo");
            }
            if (staffid.HasValue)
            {
                builder.AddCondition("cs.StaffId = @staffid");
            }
            if (busid.HasValue)
            {
                builder.AddCondition("cs.BusId = @busid");
            }
            

            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = $@"
                SELECT
                    cs.*,
                    CONCAT(s.FirstName, ' ', s.LastName) AS StaffName,
                    b.N_Z AS BusName
                FROM TOs cs
                LEFT JOIN Bus b ON b.Id = cs.BusId
                LEFT JOIN Staff s ON s.Id = cs.StaffId
                {builder.Build()}";
                
            var compilingSchedules = connection.Query<To>(querySelect, new { dateForm, dateTo, busid, staffid});
            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(compilingSchedules));
            return compilingSchedules;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
  
}
